@@ -1,9 +1,8 @@
 /*
  * fptools.c, some helper functions for getcgi.c and uu(en|de)view
  *
- * Distributed by the GNU General Public License. Use and be happy.
- * Read http://www.uni-frankfurt.de/~fp/Tools/Getcgi.html for more
- * information.                     fp@informatik.uni-frankfurt.de
+ * Distributed under the terms of the GNU General Public License.
+ * Use and be happy.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -65,7 +64,7 @@ DllEntryPoint (HINSTANCE hInstance, DWORD seginfo,
 #endif
 #endif
 
-char * fptools_id = "$Id: fptools.c,v 1.4 1996/09/10 18:45:07 fp Exp $";
+char * fptools_id = "$Id: fptools.c,v 1.2 2001/06/11 20:42:36 root Exp $";
 
 /*
  * some versions of free can't handle a NULL pointer properly
