@@ -14,24 +14,16 @@
  * GNU General Public License for more details.
  */
 
-#ifndef __UUDEVIEW_H__
-#define __UUDEVIEW_H__
+#ifndef UUDEVIEW_H__
+#define UUDEVIEW_H__
 
 /*
  * This include file features all the definitions that should
  * be externally visible. This isn't much.
  *
- * $Id: uudeview.h,v 1.5 2002/10/13 13:08:43 root Exp $
+ * $Id: uudeview.h,v 1.15 2001/06/06 18:21:47 fp Exp $
  */
 
-#ifndef _ANSI_ARGS_
-#ifdef PROTOTYPES
-#define _ANSI_ARGS_(c)	c
-#else
-#define _ANSI_ARGS_(c)	()
-#endif
-#endif
-
 /*
  * Message Types
  */
@@ -82,7 +74,6 @@
 #define BH_ENCODED	(4)	/* Binhex encoded   */
 #define PT_ENCODED	(5)	/* Plain-Text encoded (MIME) */
 #define QP_ENCODED	(6)	/* Quoted-Printable (MIME)   */
-#define YENC_ENCODED	(7)	/* yEnc encoded */
 
 /*
  * Option indices for GetOption / SetOption
@@ -107,7 +98,6 @@
 #define UUOPT_ENCEXT	(19)	/* extension for single-part encoded files */
 #define UUOPT_REMOVE    (20)    /* remove input files after decoding */
 #define UUOPT_MOREMIME  (21)    /* strict MIME adherence */
-#define UUOPT_DOTDOT	(22)	/* .. unescaping has not yet been done on input files */
 
 /*
  * Code for the "action" in the progress structure
@@ -187,79 +177,66 @@ typedef struct {
 extern "C" {
 #endif
 
-int	UUEXPORT UUInitialize		_ANSI_ARGS_((void));
-int	UUEXPORT UUGetOption		_ANSI_ARGS_((int, int *, char *, int));
-int	UUEXPORT UUSetOption		_ANSI_ARGS_((int, int, char *));
-char *	UUEXPORT UUstrerror		_ANSI_ARGS_((int));
-int	UUEXPORT UUSetMsgCallback	_ANSI_ARGS_((void *,
-						     void (*) (void *, 
-							       char *,
-							       int)));
-int	UUEXPORT UUSetBusyCallback	_ANSI_ARGS_((void *,
-						     int (*) (void *,
-							      uuprogress *),
-						     long));
-int	UUEXPORT UUSetFileCallback	_ANSI_ARGS_((void *,
-						     int (*) (void *, char *,
-							      char *, int)));
-int	UUEXPORT UUSetFNameFilter	_ANSI_ARGS_((void *,
-						     char * (*) (void *,
-								 char *)));
-int	UUEXPORT UUSetFileNameCallback	_ANSI_ARGS_((void *,
-						     char * (*) (void *,
-								 char *, char *)));
-char *	UUEXPORT UUFNameFilter		_ANSI_ARGS_((char *));
-int	UUEXPORT UULoadFile		_ANSI_ARGS_((char *, char *, int, int *));
-uulist *UUEXPORT UUGetFileListItem	_ANSI_ARGS_((int));
-int	UUEXPORT UURenameFile		_ANSI_ARGS_((uulist *, char *));
-int	UUEXPORT UUDecodeToTemp		_ANSI_ARGS_((uulist *));
-int	UUEXPORT UURemoveTemp		_ANSI_ARGS_((uulist *));
-int	UUEXPORT UUDecodeFile		_ANSI_ARGS_((uulist *, char *));
-int	UUEXPORT UUInfoFile		_ANSI_ARGS_((uulist *, void *,
-						     int (*) (void *, 
-							      char *)));
-int	UUEXPORT UUSmerge		_ANSI_ARGS_((int));
-int	UUEXPORT UUCleanUp		_ANSI_ARGS_((void));
+int	UUEXPORT UUInitialize		(void);
+int	UUEXPORT UUCleanUp		(void);
 
-int	UUEXPORT UUQuickDecode		_ANSI_ARGS_((FILE *, FILE *,
-						     char *, long));
+int	UUEXPORT UUGetOption		(int, int *, char *, int);
+int	UUEXPORT UUSetOption		(int, int, char *);
+char *	UUEXPORT UUstrerror		(int);
 
-int	UUEXPORT UUEncodeMulti		_ANSI_ARGS_((FILE *, FILE *,
-						     char *, int,
-						     char *, char *, int));
-int	UUEXPORT UUEncodePartial	_ANSI_ARGS_((FILE *, FILE *,
-						     char *, int,
-						     char *, char *,
-						     int, int, long,
-						     unsigned long*));
-int	UUEXPORT UUEncodeToStream	_ANSI_ARGS_((FILE *, FILE *,
-						     char *, int,
-						     char *, int));
-int	UUEXPORT UUEncodeToFile		_ANSI_ARGS_((FILE *, char *, int,
-						     char *, char *, long));
-int	UUEXPORT UUE_PrepSingle		_ANSI_ARGS_((FILE *, FILE *,
-						     char *, int,
-						     char *, int,
-						     char *, char *,
-						     char *, int));
-int	UUEXPORT UUE_PrepPartial	_ANSI_ARGS_((FILE *, FILE *,
-						     char *, int,
-						     char *, int,
-						     int, long, long, char *,
-						     char *, char *, int));
+int	UUEXPORT UUSetMsgCallback	(void *, void (*) (void *, char *, int));
+int	UUEXPORT UUSetBusyCallback	(void *, int (*) (void *, uuprogress *), long msecs);
+int	UUEXPORT UUSetFileCallback	(void *, int (*) (void *, char *, char *, int));
+int	UUEXPORT UUSetFNameFilter	(void *, char * (*) (void *, char *));
 
-int	UUEXPORT UUE_PrepSingleExt	_ANSI_ARGS_((FILE *, FILE *,
-						     char *, int,
-						     char *, int,
-						     char *, char *,
-						     char *, char *,
-						     int));
-int	UUEXPORT UUE_PrepPartialExt	_ANSI_ARGS_((FILE *, FILE *,
-						     char *, int,
-						     char *, int,
-						     int, long, long, char *,
-						     char *, char *, char *,
-						     int));
+char *	UUEXPORT UUFNameFilter		(char *);
+
+int	UUEXPORT UULoadFile		(char *, char *, int, int *partcount);
+uulist *UUEXPORT UUGetFileListItem	(int);
+int	UUEXPORT UURenameFile		(uulist *, char *);
+int	UUEXPORT UUDecodeToTemp		(uulist *);
+int	UUEXPORT UURemoveTemp		(uulist *);
+int	UUEXPORT UUDecodeFile		(uulist *, char *);
+int	UUEXPORT UUInfoFile		(uulist *, void *, int (*) (void *, char *));
+int	UUEXPORT UUSmerge		(int);
+
+int	UUEXPORT UUQuickDecode		(FILE *, FILE *, char *, long);
+
+int	UUEXPORT UUEncodeMulti		(FILE *, FILE *,
+				         char *, int,
+				         char *, char *, int);
+int	UUEXPORT UUEncodePartial	(FILE *, FILE *,
+				         char *, int,
+				         char *, char *,
+				         int, int, long);
+int	UUEXPORT UUEncodeToStream	(FILE *, FILE *,
+				         char *, int,
+				         char *, int);
+int	UUEXPORT UUEncodeToFile		(FILE *, char *, int,
+				         char *, char *, long);
+int	UUEXPORT UUE_PrepSingle		(FILE *, FILE *,
+				         char *, int,
+				         char *, int,
+				         char *, char *,
+				         char *, int);
+int	UUEXPORT UUE_PrepPartial	(FILE *, FILE *,
+				         char *, int,
+				         char *, int,
+				         int, long, long, char *,
+				         char *, char *, int);
+
+int    UUEXPORT UUE_PrepSingleExt       (FILE *, FILE *,
+                                         char *, int,
+                                         char *, int,
+                                         char *, char *,
+                                         char *, char *,
+                                         int);
+int    UUEXPORT UUE_PrepPartialExt      (FILE *, FILE *,
+                                         char *, int,
+                                         char *, int,
+                                         int, long, long, char *,
+                                         char *, char *, char *,
+                                         int);
 #ifdef __cplusplus
 }
 #endif
